#include <allegro.h>

#include "defines.h"
#include "bullets.h"


/**[txh]********************************************************************

  Description: Function which drops a charge from the top of the water.
  side should be zero for left side.
  
***************************************************************************/


void     drop_mine (int side)
{
   int f;
   fixed t;

   f = give_free_mine();
   if (f < 0)
      return ;

   /* Assign horizontal position fist */
   t = I2F (ship->w / 2);
   if (!side)
      t *= -1;

   MI_F.x = posx + t - I2F (load->w / 2);
   MI_F.y = I2F (ship->h*2 + 1);
   MI_F.used = 2;
   MI_F.speed = fdiv (I2F (back->h), I2F (-MINE_SPEED/4));
}

/**[txh]********************************************************************

  Description: Initializes a new mine slot with position and all that.

***************************************************************************/

void     pop_mine (fixed x, int sub_line)
{
   int f;
   fixed y;

   y = I2F (ship->h*2 + 1 + sub->h + (sub->h + 1) * sub_line);

   x -= I2F (mine->w / 2);
   y -= I2F (mine->h / 2);
   y += I2F (sub->h / 2);

   if (x < 0 || x > I2F (back->w))
      return ;

   f = give_free_mine();
   if (f < 0)
      return ;

   MI_F.used = 1;
   MI_F.x = x;
   MI_F.y = y;
   MI_F.speed = fdiv (I2F (back->h), I2F (MINE_SPEED));
}

/**[txh]********************************************************************

  Return: (int) Index to a free mine slot, or -1 if none is available.

***************************************************************************/

int      give_free_mine (void)
{
   int f;
   
   if (mines[MAX_MINES-1].used)
      return -1;

   for (f = 0; MI_F.used; f++);
   
   return f;
}

/**[txh]********************************************************************

  Description: Moves all the mines up a little bit. Also checks the
  collision against ship.

***************************************************************************/

void     move_mines (void)
{
   int f;
   
   for (f = 0; f < MAX_MINES; f++)
   {
      if (MI_F.used)
      {
         MI_F.y -= MI_F.speed;

         if (MI_F.speed > 0)
         {
            /* Check for mine->ship hit */
            if (MI_F.y < I2F (ship->h*2 + 1))
            {
               MI_F.used = 0;
      
               if (!ship_col (MI_F.x))
                  ship_col (MI_F.x + I2F (mine->w));
            }
         }
         else
         {
            /* Check for dropped mine->floor hit */
            if (MI_F.y > I2F (back->h))
               MI_F.used = 0;

            if (sub_col (MI_F.x, MI_F.y))
            {
               MI_F.used = 0;
               draw_table = 0;
            }
         }
      }
   }
}


/**[txh]********************************************************************

  Description: Detects collision from mine against ship. Simple
  unidimensional collision detection, since the y position is irrelevant.
  The function increases life...

  Return: (int) Nonzero if there was a collision.

***************************************************************************/


int      ship_col (fixed x)
{
   fixed wi;

   wi = I2F (ship->w / 2) - I2F (BORDER);

   /* Show explosion TODO */

   if (x > posx - wi && x < posx + wi)
   {
      /* Collision! */
      life ++;

      if (life == 1) MAKE_SHIP (YELLOW);
      if (life == 2) MAKE_SHIP (RED);
      
      return 1;
   }

   return 0;
}

/**[txh]********************************************************************

  Description: Detects collision between dropped mine and submarine...
  Pass top left corner of the mine.

  Return: (int) Zero if there wan't collision.

***************************************************************************/


int      sub_col (fixed x, fixed y)
{
   int f;
   fixed he = I2F (ship->h*2 + 1 + sub->h), t2 = I2F (mine->h), t1 = I2F (sub->h);
   fixed x1, x2;

   for (f = 0; f < SUB_LINES; f++, he += I2F (sub->h + 1))
   {
      if (!EN_F.used)
         continue;

      if ((y > he && y < he + t1) || (y + t2 > he && y + t2 < he + t1))
      {
         x1 = EN_F.pos - I2F (sub->w / 2);
         x2 = EN_F.pos + I2F (sub->w / 2);
         if ((x > x1 && x < x2) || (x + t2 > x1 && x + t2 < x2))
         {
            EN_F.speed = 0;
            EN_F.mines = 0;
            score += EN_F.points;
            EN_F.points = 0;
            return 1;
         }
      }
   }

   return 0;
}


