#include <allegro.h>

#include "defines.h"
#include "bullets.h"
#include "enemy.h"

/**[txh]********************************************************************

  Description: Looks if there are available sub lines.

  Return: (int) Zero if all lines are being used.

***************************************************************************/

int      free_sub_lines (void)
{
   int f;

   for (f = 0; f < SUB_LINES; f++)
      if (!EN_F.used)
         return 1;

   return 0;
}

/**[txh]********************************************************************

  Return: (int) Index to a free sub line.

***************************************************************************/

int      random_free_sub_line (void)
{
   int f;

   f = rand() % SUB_LINES;

   while (EN_F.used)
   {
      f++;
      if (f >= SUB_LINES)
         f = 0;
   }

   return f;
}


/**[txh]********************************************************************

  Description: Creates a new submarine on a new line at random, with random
  values.

***************************************************************************/

void     add_sub (void)
{
   BITMAP *num;
   int f, w, h;
   char text[10];
   fixed temp;
   
   f = random_free_sub_line();

   level++;
   EN_F.used = 1;
   EN_F.mines = 10 + level / 2;
   EN_F.time = 0;
   EN_F.speed = fdiv (deltax, I2F (1 + rand()%16));
   EN_F.points = F2I (fmul (fdiv (I2F (MAX_SUB_POINTS), deltax), EN_F.speed));
   EN_F.points += f + rand() % 10;
   
   if (rand()%2)
   {
      EN_F.speed *= -1;
      EN_F.pos = I2F (back->w + sub->w / 2);
   }
   else
      EN_F.pos = I2F (- sub->w / 2);

   /* Bitmap creation... */

   if (!EN_F.bmp)
   {
      EN_F.bmp = CB (sub->w, sub->h);
      if (!EN_F.bmp)
         goto abort;
   }

   usprintf (text, "%d", EN_F.points);

   num = CB (text_length (font, text), text_height (font));
   if (!num)
   {
      abort:
      SHOW_OUT ("not_enough_mem");
      allegro_exit();
      exit (69);
   }

   clear (EN_F.bmp);
   clear (num);
   textout (num, font, text, 0, 0, BLACK);
   draw_sprite (EN_F.bmp, sub, 0, 0);
   
   temp = fdiv (I2F (EN_F.bmp->h - EN_F.bmp->h/4), I2F (num->h));
   w = F2I (num->w * temp);
   h = F2I (num->h * temp);
   stretch_sprite (EN_F.bmp, num, EN_F.bmp->w / 2 - w / 2, EN_F.bmp->h / 2 - h / 2,
      w, h);

   (EN_F.speed > 0) ? (w = 0) : (w = EN_F.bmp->w-3);
   vline (EN_F.bmp, w, 0, EN_F.bmp->h-1, WHITE);
   w++;
   vline (EN_F.bmp, w, 0, EN_F.bmp->h-1, WHITE);
}

/**[txh]********************************************************************

  Description: Takes care of moving all the submarines.

***************************************************************************/

void     move_subs (void)
{
   int f;

   for (f = 0; f < SUB_LINES; f++)
   {
      if (!EN_F.used)
         continue;

      EN_F.pos += EN_F.speed;

      if (!EN_F.speed)  EN_F.time ++;

      /* Do we destroy subs? */
      if (EN_F.pos < I2F (- sub->w / 2) || EN_F.time > SUB_DEATH ||
         EN_F.pos > I2F (back->w + sub->w / 2))
         EN_F.used = 0;

      if (rand()%10000 < EN_F.mines)
         pop_mine (EN_F.pos, f);
   }
}


